"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var react_1 = __importStar(require("react"));
var react_native_1 = require("react-native");
var react_native_gesture_handler_1 = require("react-native-gesture-handler");
var react_native_reanimated_1 = __importStar(require("react-native-reanimated"));
var react_native_redash_1 = require("react-native-redash");
var Segment_1 = require("../Segment");
var utils_1 = require("../utils");
var SegmentedControlStyles_1 = __importDefault(require("./SegmentedControlStyles"));
exports.SegmentedContext = react_1.createContext(null);
exports.SegmentedControl = function (_a) {
    var _b = _a.activeTintColor, activeTintColor = _b === void 0 ? '#000000' : _b, children = _a.children, _c = _a.inactiveTintColor, inactiveTintColor = _c === void 0 ? '#000000' : _c, initialSelectedName = _a.initialSelectedName, onChangeValue = _a.onChangeValue, sliderStyle = _a.sliderStyle, style = _a.style;
    var _d = react_1.useState(false), _initialized = _d[0], _setInitialized = _d[1];
    var _e = react_1.useState(0), _width = _e[0], _setWidth = _e[1];
    var _initialSelectedName = react_1.useState(initialSelectedName)[0];
    var _f = react_1.useState(_initialSelectedName), _activeName = _f[0], _setActiveName = _f[1];
    var _g = react_1.useState(new react_native_reanimated_1.default.Value(0)), _sliderPosition = _g[0], _setSliderPosition = _g[1];
    var _h = react_1.useState(0), _sliderWidth = _h[0], _setSliderWidth = _h[1];
    var _j = react_1.useState(undefined), _map = _j[0], _setMap = _j[1];
    var values = Array.isArray(children) ? children : [children];
    // Map segment names to index
    react_1.useEffect(function () {
        var tempMap = {};
        values.forEach(function (child, index) {
            if (child.type !== Segment_1.Segment) {
                throw new Error('SegmentedControl only accepts Segment as children.');
            }
            if (!child.props.name) {
                throw new Error('Segment requires `name` to be defined.');
            }
            tempMap[child.props.name] = index;
        });
        _setMap(tempMap);
    }, []);
    // Set slider width
    react_1.useEffect(function () {
        _setSliderWidth(_width * (1 / values.length - 0.015));
    }, [values, _width]);
    // Set initial slider position
    react_1.useEffect(function () {
        if (typeof _initialSelectedName !== 'undefined' &&
            typeof _map !== 'undefined' &&
            _width > 0 &&
            !_initialized) {
            var index = _map[_initialSelectedName];
            var position = _width * (index / values.length);
            _setSliderPosition(new react_native_reanimated_1.default.Value(position));
            _setInitialized(true);
        }
    }, [values, _width, _map, _initialSelectedName]);
    // This hook is used to animate the slider position
    react_native_reanimated_1.default.useCode(function () {
        var index = _activeName && _map ? _map[_activeName] : 0;
        var sliderPosition = _width * (index / values.length);
        return react_native_reanimated_1.default.set(_sliderPosition, react_native_redash_1.timing({
            from: _sliderPosition,
            to: sliderPosition,
            easing: react_native_reanimated_1.Easing.linear,
            duration: 200,
        }));
    }, [_activeName]);
    var handleLayout = function (event) {
        return _setWidth(event.nativeEvent.layout.width);
    };
    var handleChangeValue = function (name) {
        if (typeof _activeName === 'undefined' && typeof _map !== 'undefined') {
            var index = _map[name];
            _setSliderPosition(new react_native_reanimated_1.default.Value(_width * (index / values.length)));
        }
        _setActiveName(name);
        if (typeof onChangeValue === 'function') {
            onChangeValue(name);
        }
    };
    var handleGestureEvent = function (event) {
        var x = event.nativeEvent.x;
        var calculatedIndex = Math.floor((x / _width) * values.length);
        var index = utils_1.clamp(calculatedIndex, 0, values.length - 1);
        var name = values[index].props.name;
        handleChangeValue(name);
    };
    return (react_1.default.createElement(exports.SegmentedContext.Provider, { value: { selectedName: _activeName, onChange: handleChangeValue } },
        react_1.default.createElement(react_native_gesture_handler_1.PanGestureHandler, { onGestureEvent: handleGestureEvent },
            react_1.default.createElement(react_native_1.View, { onLayout: handleLayout, style: [SegmentedControlStyles_1.default.container, style] },
                typeof _activeName !== 'undefined' && (react_1.default.createElement(react_native_reanimated_1.default.View, { testID: "SegmentedControl_Slider", style: [
                        SegmentedControlStyles_1.default.sliderDefault,
                        {
                            width: _sliderWidth,
                            transform: [
                                {
                                    translateX: _sliderPosition,
                                },
                            ],
                        },
                        sliderStyle,
                        SegmentedControlStyles_1.default.slider,
                    ] })),
                values.map(function (child, index) { return (react_1.default.createElement(react_1.default.Fragment, { key: child.props.name },
                    index > 0 && (react_1.default.createElement(react_native_1.View, { style: SegmentedControlStyles_1.default.dividerContainer },
                        react_1.default.createElement(react_native_1.View, { style: SegmentedControlStyles_1.default.divider }))), __assign(__assign({}, child), { props: __assign(__assign({}, child.props), { inactiveTintColor: inactiveTintColor,
                        activeTintColor: activeTintColor }) }))); })))));
};
exports.default = exports.SegmentedControl;
//# sourceMappingURL=SegmentedControl.js.map